﻿namespace ExplorerBarReorder
{
    using System.Windows.Forms;
    using Infragistics.Win.UltraWinToolbars;

    /// <summary>
    /// Main Form
    /// </summary>
    /// <seealso cref="System.Windows.Forms.Form" />
    public partial class Form1 : Form
    {
        /// <summary>
        /// The explorer bar item
        /// </summary>
        private Infragistics.Win.UltraWinExplorerBar.UltraExplorerBarItem explorerBarItem;

        /// <summary>
        /// Initializes a new instance of the <see cref="Form1"/> class.
        /// </summary>
        public Form1()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Builds the context menu.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="location">The location.</param>
        /// <param name="element">The element.</param>
        private void BuildContextMenu(object sender, System.Drawing.Point location, Infragistics.Win.UIElement element)
        {
            // Only add if we are on an explorer bar item
            if (element != null && element.SelectableItem is Infragistics.Win.UltraWinExplorerBar.UltraExplorerBarItem)
            {
                // Move up was added
                bool moveUpAdded = false;

                // Clear tools
                this.contextMenuToolbarsManager.Tools.Clear();
                PopupMenuTool root = new PopupMenuTool("ROOT");
                this.contextMenuToolbarsManager.Tools.Add(root);

                Infragistics.Win.UltraWinExplorerBar.UltraExplorerBarItem selectedItem = (Infragistics.Win.UltraWinExplorerBar.UltraExplorerBarItem)element.SelectableItem;

                // Update explorer bar item
                this.explorerBarItem = selectedItem.Clone();

                // Move Up
                if (selectedItem.Index > 0)
                {
                    ButtonTool moveUpTool = new ButtonTool("Move Up");

                    moveUpTool.SharedProps.Caption = "Move Up";
                    moveUpTool.SharedProps.DisplayStyle = Infragistics.Win.UltraWinToolbars.ToolDisplayStyle.ImageAndText;

                    moveUpTool.ToolClick += new ToolClickEventHandler(this.MoveUp_ToolClick);

                    this.contextMenuToolbarsManager.Tools.Add(moveUpTool);
                    root.Tools.AddTool("Move Up");

                    // Has to be set after the tool is added.
                    root.Tools["Move Up"].InstanceProps.IsFirstInGroup = true;

                    // Added Separator
                    moveUpAdded = true;
                }

                // Move Down
                if (selectedItem.Index < this.ultraExplorerBar.Groups["Sessions"].Items.Count - 1)
                {
                    ButtonTool moveDownTool = new ButtonTool("Move Down");

                    moveDownTool.SharedProps.Caption = "Move Down";
                    moveDownTool.SharedProps.DisplayStyle = Infragistics.Win.UltraWinToolbars.ToolDisplayStyle.ImageAndText;

                    moveDownTool.ToolClick += new ToolClickEventHandler(this.MoveDown_ToolClick);

                    this.contextMenuToolbarsManager.Tools.Add(moveDownTool);
                    root.Tools.AddTool("Move Down");

                    // Make it the first in group if move up was not added
                    if (moveUpAdded == false)
                    {
                        // Has to be set after the tool is added.
                        root.Tools["Move Down"].InstanceProps.IsFirstInGroup = true;
                    }
                }

                this.contextMenuToolbarsManager.DockWithinContainer = this.ultraExplorerBar;
                this.contextMenuToolbarsManager.ShowPopup("ROOT", (Control)sender, ((Control)sender).PointToScreen(location));
            }
        }

        /// <summary>
        /// Handles the ToolClick event of the MoveDown control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="ToolClickEventArgs"/> instance containing the event data.</param>
        private void MoveDown_ToolClick(object sender, ToolClickEventArgs e)
        {
            int index = this.ultraExplorerBar.Groups["Sessions"].Items[this.explorerBarItem.Key].Index;

            this.ultraExplorerBar.Groups["Sessions"].Items.Remove(this.explorerBarItem);
            this.ultraExplorerBar.Groups["Sessions"].Items.Insert(index + 1, this.explorerBarItem);
        }

        /// <summary>
        /// Handles the ToolClick event of the MoveUp control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="ToolClickEventArgs"/> instance containing the event data.</param>
        private void MoveUp_ToolClick(object sender, ToolClickEventArgs e)
        {
            int index = this.ultraExplorerBar.Groups["Sessions"].Items[this.explorerBarItem.Key].Index;

            this.ultraExplorerBar.Groups["Sessions"].Items.Remove(this.explorerBarItem);
            this.ultraExplorerBar.Groups["Sessions"].Items.Insert(index - 1, this.explorerBarItem);
        }

        /// <summary>
        /// Handles the MouseUp event of the UltraExplorerBar control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="MouseEventArgs"/> instance containing the event data.</param>
        private void UltraExplorerBar_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                // Time to build the context menu based on the active row.
                this.BuildContextMenu(sender, e.Location, this.ultraExplorerBar.UIElement.ElementFromPoint(e.Location));
            }
        }
    }
}